==> trayplugin1.2.zip: trayplugin1.2/source/tray.c <==
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <stdio.h>

#include "tray.h"
#include "common/utils.h"
#include "intl.h"

#define SYSTRAY_HINT _("Sylpheed Tray icon")
#define WM_TRAYMESSAGE WM_USER

enum _SYSTRAY_CMND {
	SYSTRAY_CMND_MENU_EXIT,
	SYSTRAY_CMND_HIDE,
	SYSTRAY_CMND_SHOW
};
typedef enum _SYSTRAY_CMND SYSTRAY_CMND;

static GtkWidget *MainWin;
static HWND mainwin_hwnd;
static HINSTANCE this_hinstance;

static HWND systray_hwnd=0;
static HICON systray_icon=0;
static NOTIFYICONDATA sysicon_nid;
static HMENU systray_menu=0;

static long oldwndproc;

static LRESULT CALLBACK systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

static void MinimizeToTray(HWND hWnd);
static void RestoreFromTray(HWND hWnd);

/* -------------------------------------------------------------------------
 * Create Tray window
 */

static HWND systray_create_hiddenwin() {
	WNDCLASS wc;
	TCHAR wname[32];

	ZeroMemory(&wc, sizeof(wc));
	strcpy(wname, "HiddenSylpheedTrayWindow56789012");
	wc.lpfnWndProc	 = (WNDPROC)systray_mainmsg_handler;
	wc.hInstance	 = this_hinstance;
	wc.lpszClassName = wname;
	RegisterClass(&wc);
	return (CreateWindow(wname, "", 0, 0, 0, 0, 0, 
		GetDesktopWindow(), NULL, this_hinstance, 0));
}

/* -------------------------------------------------------------------------
 * Create / Show Tray menu
 */

static void systray_create_menu(void) {
	if((systray_menu = CreatePopupMenu())) {
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_SHOW, _("Show")))
			debug_print("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_HIDE, _("Hide")))
			debug_print("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_SEPARATOR, 0, 0))
			debug_print("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_MENU_EXIT, _("Exit")))
			debug_print("AppendMenu error: %d\n", GetLastError());
	} else
		debug_print("CreatePopupMenu error: %d\n", GetLastError());
}

static void systray_show_menu(int x, int y) {
	SetForegroundWindow(systray_hwnd);
	TrackPopupMenu(systray_menu,         // handle to shortcut menu
		       TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_LEFTBUTTON,
		       x,                   // horizontal position, in screen coordinates
		       y,                   // vertical position, in screen coordinates
		       0,                   // reserved, must be zero
		       systray_hwnd,        // handle to owner window
		       NULL                 // ignored
		       );
}

/* -------------------------------------------------------------------------
 * Set up Tray icons
 */

static void systray_init_icon(HWND hWnd, HICON icon) {
	ZeroMemory(&sysicon_nid,sizeof(sysicon_nid));
	sysicon_nid.cbSize=sizeof(NOTIFYICONDATA);
	sysicon_nid.hWnd=hWnd;
	sysicon_nid.uFlags=NIF_ICON | NIF_MESSAGE | NIF_TIP;
	sysicon_nid.uCallbackMessage=WM_TRAYMESSAGE;
	sysicon_nid.hIcon=icon;
	strcpy(sysicon_nid.szTip,SYSTRAY_HINT);
}

/* -------------------------------------------------------------------------
 * Tray menu message handler
 */

static LRESULT CALLBACK systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam) {

	switch(msg) {
	case WM_COMMAND:
		debug_print("WM_COMMAND\n");
		switch(LOWORD(wparam)) {
		case SYSTRAY_CMND_MENU_EXIT:
			SendMessage(mainwin_hwnd, WM_CLOSE, 0, 0);
			break;
		case SYSTRAY_CMND_SHOW:
			debug_print("Show\n");
			RestoreFromTray(mainwin_hwnd);
			break;
		case SYSTRAY_CMND_HIDE:
			MinimizeToTray(mainwin_hwnd);
			debug_print("Hide\n");
			break;
		}
		break;
	case WM_TRAYMESSAGE:
	{
		if( lparam == WM_LBUTTONDBLCLK ) {
			
			if (IsWindowVisible(mainwin_hwnd))
				MinimizeToTray(mainwin_hwnd);
			else
				RestoreFromTray(mainwin_hwnd);
			debug_print("Systray got double click\n");
		}
		if( lparam == WM_RBUTTONUP ) {
			POINT mpoint;
			GetCursorPos(&mpoint);
			systray_show_menu(mpoint.x, mpoint.y);
		}
		break;
	}
	default: ;
	}/* end switch */

	return DefWindowProc(hwnd, msg, wparam, lparam);
}

/* -------------------------------------------------------------------------
 * Tray window message handler
 */

static LRESULT CALLBACK mainwndproc_handler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	boolean handled=TRUE;

	switch (uMsg) {
		case WM_SYSCOMMAND: {
			switch (wParam) {
				case SC_MINIMIZE: MinimizeToTray(mainwin_hwnd);	break;
				default: handled=FALSE;
			}
		} ; break;
		case WM_CLOSE: tray_close(); handled=FALSE; break;
		default: handled=FALSE;
	}

	if (handled)
		return 0;
	else
		return CallWindowProc((WNDPROC)oldwndproc, hWnd, uMsg, wParam, lParam);
}

/* -------------------------------------------------------------------------
 * Minimize / Restore (internal use)
 */

static void MinimizeToTray(HWND hWnd) {
	if (trayplugin_enable)
		ShowWindow(hWnd,SW_HIDE);
	else
		ShowWindow(hWnd,SW_MINIMIZE);
	ShowTrayIcon(trayplugin_enable);
}

static void RestoreFromTray(HWND hWnd) {
	ShowWindow(hWnd,SW_SHOW);
	SetActiveWindow(hWnd);
	SetForegroundWindow(hWnd);
	if (!trayplugin_persistent)
		ShowTrayIcon(FALSE);
}

/* -------------------------------------------------------------------------
 * Exported functions (init, destroy, show/hide icon, minimize/maximize)
 */

extern void SetWindowState(gint winstate) {
	switch (winstate) {
	case TRAY_STATE_NONE:		break;
	case TRAY_STATE_RESTORE:	break;
	case TRAY_STATE_MINIMIZED:	MinimizeToTray(mainwin_hwnd); break;
	case TRAY_STATE_MAXIMIZED:	ShowWindow(mainwin_hwnd, SW_MAXIMIZE); break;
	}
}

extern void ShowTrayIcon(boolean show) {
	if (show)
		Shell_NotifyIcon(NIM_ADD,&sysicon_nid);
	else
		Shell_NotifyIcon(NIM_DELETE,&sysicon_nid);
}

extern int tray_close() {
	SetWindowLong(mainwin_hwnd, GWL_WNDPROC, oldwndproc);
	ShowTrayIcon(FALSE);
	DestroyWindow(systray_hwnd);
	return 1;
}

extern int tray_init(GtkWidget *mainwin) {
	MainWin = mainwin;
	this_hinstance = GetModuleHandle(NULL);
	mainwin_hwnd=GDK_DRAWABLE_XID(mainwin->window);
	oldwndproc = GetWindowLong(mainwin_hwnd, GWL_WNDPROC);
	SetWindowLong(mainwin_hwnd, GWL_WNDPROC, (long)mainwndproc_handler);
	systray_hwnd = systray_create_hiddenwin();
	systray_create_menu();
	systray_icon = LoadIcon(this_hinstance,MAKEINTRESOURCE(IDI_ICON1));
	systray_init_icon(systray_hwnd, systray_icon);
	ShowTrayIcon(trayplugin_enable & trayplugin_persistent);
	return 1;
}

==> trayplugin1.4.1.zip: trayplugin1.4.1/source/tray.c <==
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <stdio.h>

#include "tray.h"
#include "common/utils.h"
#include "intl.h"

#define SYSTRAY_HINT _("Sylpheed Tray icon")
#define WM_TRAYMESSAGE WM_USER

enum _SYSTRAY_CMND {
	SYSTRAY_CMND_MENU_EXIT,
	SYSTRAY_CMND_HIDE,
	SYSTRAY_CMND_SHOW
};
typedef enum _SYSTRAY_CMND SYSTRAY_CMND;

static GtkWidget *MainWin;
static HWND mainwin_hwnd;
static HINSTANCE this_hinstance;

static HWND systray_hwnd=0;
static HICON systray_icon=0;
static NOTIFYICONDATA sysicon_nid;
static HMENU systray_menu=0;
static HICON icon_unread, icon_new, icon_default;

static long oldwndproc;

static LRESULT CALLBACK systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

static void MinimizeToTray(HWND hWnd);
static void RestoreFromTray(HWND hWnd);

/* -------------------------------------------------------------------------
 * Create Tray window
 */

static HWND systray_create_hiddenwin(void) {
	WNDCLASS wc;
	TCHAR wname[32];

	ZeroMemory(&wc, sizeof(wc));
	strcpy(wname, "HiddenSylpheedTrayWindow56789012");
	wc.lpfnWndProc	 = (WNDPROC)systray_mainmsg_handler;
	wc.hInstance	 = this_hinstance;
	wc.lpszClassName = wname;
	RegisterClass(&wc);
	return (CreateWindow(wname, "", 0, 0, 0, 0, 0, 
		GetDesktopWindow(), NULL, this_hinstance, 0));
}

/* -------------------------------------------------------------------------
 * Create / Show Tray menu
 */

static void systray_create_menu(void) {
	if((systray_menu = CreatePopupMenu())) {
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_SHOW, _("Show")))
			debug_print("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_HIDE, _("Hide")))
			debug_print("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_SEPARATOR, 0, 0))
			debug_print("AppendMenu error: %d\n", GetLastError());
		if(!AppendMenu(systray_menu, MF_STRING, SYSTRAY_CMND_MENU_EXIT, _("Exit")))
			debug_print("AppendMenu error: %d\n", GetLastError());
	} else
		debug_print("CreatePopupMenu error: %d\n", GetLastError());
}

static void systray_show_menu(int x, int y) {
	SetForegroundWindow(systray_hwnd);
	TrackPopupMenu(systray_menu,         // handle to shortcut menu
		       TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_LEFTBUTTON,
		       x,                   // horizontal position, in screen coordinates
		       y,                   // vertical position, in screen coordinates
		       0,                   // reserved, must be zero
		       systray_hwnd,        // handle to owner window
		       NULL                 // ignored
		       );
}

/* -------------------------------------------------------------------------
 * Set up Tray icons
 */

static void systray_init_icon(HWND hWnd, HICON icon) {
	ZeroMemory(&sysicon_nid,sizeof(sysicon_nid));
	sysicon_nid.cbSize=sizeof(NOTIFYICONDATA);
	sysicon_nid.hWnd=hWnd;
	sysicon_nid.uFlags=NIF_ICON | NIF_MESSAGE | NIF_TIP;
	sysicon_nid.uCallbackMessage=WM_TRAYMESSAGE;
	sysicon_nid.hIcon=icon;
	strcpy(sysicon_nid.szTip,SYSTRAY_HINT);
}

/* -------------------------------------------------------------------------
 * Tray menu message handler
 */

static LRESULT CALLBACK systray_mainmsg_handler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam) {

	switch(msg) {
	case WM_COMMAND:
		debug_print("WM_COMMAND\n");
		switch(LOWORD(wparam)) {
		case SYSTRAY_CMND_MENU_EXIT:
			SendMessage(mainwin_hwnd, WM_CLOSE, 0, 0);
			break;
		case SYSTRAY_CMND_SHOW:
			debug_print("Show\n");
			RestoreFromTray(mainwin_hwnd);
			break;
		case SYSTRAY_CMND_HIDE:
			MinimizeToTray(mainwin_hwnd);
			debug_print("Hide\n");
			break;
		}
		break;
	case WM_TRAYMESSAGE:
	{
		if( lparam == WM_LBUTTONDBLCLK ) {
			
			if (IsWindowVisible(mainwin_hwnd))
				MinimizeToTray(mainwin_hwnd);
			else
				RestoreFromTray(mainwin_hwnd);
			debug_print("Systray got double click\n");
		}
		if( lparam == WM_RBUTTONUP ) {
			POINT mpoint;
			GetCursorPos(&mpoint);
			systray_show_menu(mpoint.x, mpoint.y);
		}
		break;
	}
	default: ;
	}/* end switch */

	return DefWindowProc(hwnd, msg, wparam, lparam);
}

/* -------------------------------------------------------------------------
 * Tray window message handler
 */

static LRESULT CALLBACK mainwndproc_handler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	boolean handled=TRUE;

	switch (uMsg) {
		case WM_SYSCOMMAND: {
			switch (wParam) {
				case SC_MINIMIZE: MinimizeToTray(mainwin_hwnd);	break;
				default: handled=FALSE;
			}
		} ; break;
		case WM_CLOSE: tray_close(); handled=FALSE; break;
		default: handled=FALSE;
	}

	if (handled)
		return 0;
	else
		return CallWindowProc((WNDPROC)oldwndproc, hWnd, uMsg, wParam, lParam);
}

/* -------------------------------------------------------------------------
 * Minimize / Restore (internal use)
 */

static void MinimizeToTray(HWND hWnd) {
	if (trayplugin_enable)
		ShowWindow(hWnd,SW_HIDE);
	else
		ShowWindow(hWnd,SW_MINIMIZE);
	ShowTrayIcon(trayplugin_enable);
}

static void RestoreFromTray(HWND hWnd) {
	ShowWindow(hWnd,SW_SHOW);
	SetActiveWindow(hWnd);
	SetForegroundWindow(hWnd);
	if (!trayplugin_persistent)
		ShowTrayIcon(FALSE);
}

/* -------------------------------------------------------------------------
 * Exported functions (init, destroy, show/hide icon, minimize/maximize)
 */

extern void SetWindowState(gint winstate) {
	switch (winstate) {
	case TRAY_STATE_NONE:		break;
	case TRAY_STATE_RESTORE:	break;
	case TRAY_STATE_MINIMIZED:	MinimizeToTray(mainwin_hwnd); break;
	case TRAY_STATE_MAXIMIZED:	ShowWindow(mainwin_hwnd, SW_MAXIMIZE); break;
	}
}

extern void ShowTrayIcon(boolean show) {
	if (show)
		Shell_NotifyIcon(NIM_ADD,&sysicon_nid);
	else
		Shell_NotifyIcon(NIM_DELETE,&sysicon_nid);
}

extern int tray_set_hint(const char *hint)
{
	strcpy(sysicon_nid.szTip, hint);
	Shell_NotifyIcon(NIM_MODIFY,&sysicon_nid);
	return 1;
}

extern int tray_set_icon(int icon)
{
	switch (icon) {
		case ICON_UNREAD: 
			sysicon_nid.hIcon=icon_unread; break;
		case ICON_NEW: 
			sysicon_nid.hIcon=icon_new; break;
		default: 
			sysicon_nid.hIcon=icon_default; break;
	}
	Shell_NotifyIcon(NIM_MODIFY,&sysicon_nid);
	return 1;
}

extern int tray_close(void) {
	SetWindowLong(mainwin_hwnd, GWL_WNDPROC, oldwndproc);
	ShowTrayIcon(FALSE);
	DestroyWindow(systray_hwnd);
	return 1;
}

extern int tray_init(GtkWidget *mainwin) {
	char buf[4096]={0};
	char *fname = buf;

	MainWin = mainwin;
	this_hinstance = GetModuleHandle(NULL);
	mainwin_hwnd=GDK_DRAWABLE_XID(mainwin->window);
	oldwndproc = GetWindowLong(mainwin_hwnd, GWL_WNDPROC);
	SetWindowLong(mainwin_hwnd, GWL_WNDPROC, (long)mainwndproc_handler);

	systray_hwnd = systray_create_hiddenwin();
	systray_create_menu();
	systray_icon = LoadIcon(this_hinstance,MAKEINTRESOURCE(IDI_ICON1));

	if (GetModuleFileName(0, buf, sizeof(buf)))
		fname=strrchr(buf, '\\');

	strcpy(fname, "\\unread.ico");
	if (!(icon_unread = LoadImage( 0, buf, IMAGE_ICON, 0,0, LR_LOADFROMFILE)))
		icon_unread	= systray_icon;
	strcpy(fname, "\\new.ico");
	if (!(icon_new    = LoadImage( 0, buf, IMAGE_ICON, 0,0, LR_LOADFROMFILE)))
		icon_new	= systray_icon;
	strcpy(fname, "\\default.ico");
	if (!(icon_default = LoadImage( 0, buf, IMAGE_ICON, 0,0, LR_LOADFROMFILE)))
		icon_default	= systray_icon;

	systray_init_icon(systray_hwnd, systray_icon);
	ShowTrayIcon(trayplugin_enable & trayplugin_persistent);
	return 1;
}
